/*
 *  stereo -- Stereo matching software
 *  $Id: defs.h 305 2004-09-09 13:12:42Z sam $
 *  Copyright (c) 2004 Sam Hocevar <sam@zoy.org>
 *                     Éric Beets <ericbeets@free.fr>
 *  All Rights Reserved
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA
 */

#ifndef __DEFS_H__
#define __DEFS_H__

#include <stdint.h>
#include <stdlib.h>
#include <string.h>

/* Tracker behaviour */
#define TRACKV1	1
#define TRACKV2	2
#define TRACKV3	4
#define TRACKV4	8
extern int track_mode;
extern int size_mode;

/* Array handling */
#define array_append(array, item) \
    if((array) == NULL) \
    { \
        array = malloc(sizeof(array[0])); \
        array[0] = item; \
        array##_count = 1; \
    } \
    else \
    { \
        unsigned int count = array##_count; \
        while(!(count & 1)) \
            count >>= 1; \
        if(count == 1) \
            array = realloc(array, 2 * array##_count * sizeof(array[0])); \
        array[array##_count] = item; \
        array##_count++; \
    }

#define array_insert(array, item, pos) \
    if((array) == NULL) \
    { \
        array = malloc(sizeof(array[0])); \
        array[0] = item; \
        array##_count = 1; \
    } \
    else \
    { \
        unsigned int count = array##_count; \
        while(!(count & 1)) \
            count >>= 1; \
        if(count == 1) \
            array = realloc(array, 2 * array##_count * sizeof(array[0])); \
        memmove(array + pos + 1, array + pos, array##_count - pos); \
        array[pos] = item; \
        array##_count++; \
    }

#define array_del(array, pos) \
    if((array##_count) == 1) \
    { \
        free(array); \
        array = NULL; \
        array##_count = 0; \
    } \
    else \
    { \
        unsigned int count = array##_count - 1; \
        memmove(array + pos, array + pos + 1, array##_count - pos); \
        while(!(count & 1)) \
            count >>= 1; \
        if(count == 1) \
            array = realloc(array, (array##_count - 1) * sizeof(array[0])); \
        array##_count--; \
    }

/* Primitive structs */
typedef struct _pt2d
{
    double x, y;
}
pt2d_t;

typedef struct _pt3d
{
    double x, y, z;
}
pt3d_t;

/* Higher level structs */
typedef struct _ptsing
{
    pt2d_t point;
    double score;
}
ptsing_t;

typedef struct _scanline
{
    pt2d_t a;
    pt2d_t b;
}
scanline_t;

#define IMAGE_NORMAL 1
#define IMAGE_OPENCV 2
#define IMAGE_OPENCV_HEADER 3

typedef struct _image
{
    unsigned int width, height, pitch;
    unsigned int depth;
    uint8_t * pixels;

    int _type;
    void * _private;
}
image_t;

typedef struct _projection
{
    pt2d_t intersect;
    pt3d_t vx, vy, vz, O, Os, I;
    double d, D, focal;

    double top, bottom, left, right;

    ptsing_t * ptsing;
    unsigned int ptsing_count;

    scanline_t * scanline; 
    unsigned int scanline_count;
}
projection_t;

#define RESOLUTION 1
#define THRESHOLD 400

#define DELTA_X 80
#define DELTA_Y 1

#define CORR_X 3
#define CORR_Y 3

/* Continuity weight (16 == normal) */
#define X_WEIGHT 16*0
#define Y_WEIGHT 8*0

/* Computed macros */
#define THRESH (THRESHOLD*CORR_X*CORR_Y)
#define TRANS_X ((DELTA_X*RESOLUTION)+1)
#define TRANS_Y DELTA_Y
#define WINTRANS_X (CORR_X+TRANS_X-1)
#define WINTRANS_Y (CORR_Y+TRANS_Y-1)

#define REFER_WIDTH (2048*RESOLUTION)
#define TRANS_WIDTH (2048*RESOLUTION)

typedef struct _pixeldepth
{
    int score;
    int depth;
    int depth1;
    int depth2;
    int corr;
    int corr1;
    int corr2;

    int invalid;
    int highlight;

//    int *tabcorr;
    int tabcorr[TRANS_X];
    struct _pixeldepth *next;
}
pixeldepth_t;

typedef struct _map
{
    pixeldepth_t *elem;
    int width,height;
}
map_t;

typedef struct _window
{
    int r,g,b;
}
window_t;

typedef struct _correlation
{
//  image_t line;
    map_t *Map; 
}
correlation_t;

#define LEFT 0
#define RIGHT 1

typedef struct _pair
{
    image_t image[2];
    projection_t projection[2];
    image_t image_norm[2];

    image_t depthmap_norm[2];
    image_t depthmap[2];

    correlation_t correlation[2];
    
    double adjust;
}
pair_t;

/* Fast pixel stuff */
static inline int getpixel_fast(image_t *image, int x, int y, int *r, int *g, int *b)
{
    if(x < 0 || y < 0 || x >= image->width || y >= image->height)
    {
        *r = 255;
        *g = 0;
        *b = 255;
	return -1;
    }

    *b = image->pixels[y * image->pitch + x * image->depth];
    *g = image->pixels[y * image->pitch + x * image->depth + 1];
    *r = image->pixels[y * image->pitch + x * image->depth + 2];

    return 0;
}

static inline int setpixel_fast(image_t *image, int x, int y, int r, int g, int b)
{
    if(x < 0 || y < 0 || x >= image->width || y >= image->height)
        return -1;

    image->pixels[y * image->pitch + x * image->depth] = b;
    image->pixels[y * image->pitch + x * image->depth + 1] = g;
    image->pixels[y * image->pitch + x * image->depth + 2] = r;

    return 0;
}

static inline int sq(int x)
{
    return x * x;
}

static inline int min(int i, int j)
{
    return i < j ? i : j;
}

static inline int max(int i, int j)
{
    return i > j ? i : j;
}

#endif
